//importing required D3 packages
import {
  select,
  selectAll,
  scaleOrdinal
} from 'd3';
// declaring variables 
const svg = select('svg');
const width = +svg.attr('width'); 
const height = +svg.attr('height');
const centre_x = width / 2;
const centre_y = height / 2;
const colorScale = ["red","lightgreen","blue", "gold", "orange", "aqua"];
// altering the SVG dimensions 
svg.attr("preserveAspectRatio", "xMidYMid meet")
   .attr("viewBox", [0, 0, width * 2, height * 2]);

//imporing data from json file
d3.json('data.json')
  .then(data => {
  
  const links = data.links.map(d => Object.create(d));
  links.forEach((d, i) => {
//    console.log(data.links[i]);
    d.srcType = data.links[i].srcType;
    d.tgtType = data.links[i].tgtType;
    d.relationship = data.links[i].relationship;
    d.value = data.links[i].value;
  });
  const nodes = data.nodes.map(d => Object.create(d));
  nodes.forEach((d, i) => {
  	d.group = data.nodes[i].group;
  });

  const simulation = d3.forceSimulation(nodes)
      .force("link", d3.forceLink(links)
             .id(d => d.id)
             .distance(120))
      .force("charge", d3.forceManyBody()
             .strength(-500)
//             .distanceMax(200)
            )
 // 		.force("x", d3.forceX())
 // 		.force("y", d3.forceY())
  		.force("collision", d3.forceCollide()
             	.radius(35)
            )
      .force("center", d3.forceCenter(width, height))
  ;

  const drag = simulation => {
    function dragstarted(event) {
      if (!event.active) simulation.alphaTarget(0.5).restart();
      event.subject.fx = event.subject.x;
      event.subject.fy = event.subject.y;
//      d3.select(this).classed("fixed", true);
   //   console.log(d);
    }
    function dragged(event) {
      event.subject.fx = event.x;
      event.subject.fy = event.y;
    }
    function dragended(event, d) {
      if (!event.active) simulation.alphaTarget(0);
      event.subject.fx = null;
      event.subject.fy = null;
    }
    return d3.drag()
      .on("start", dragstarted)
      .on("drag", dragged)
      .on("end", dragended);
  }
// creating and appending links
  const link = svg.append("g")
  .selectAll("line")
    .data(links)
  	.join("line")
  		.attr("marker-end", "url(#node)");
  //creating and appending nodes 
  const node = svg.append("g")
    .attr("class", "node")
  .selectAll("g")
    .data(nodes)
  	.enter()
  	.append("g")
    	.attr("transform", d => "translate(" + centre_x + ", " + centre_y + ")");
;
  // appending circles and visualizing them 
  const circles = node.append("circle")
  		.attr("r", 35)
  		.attr("fill", d => colorScale[d.group])
			.on("mouseover", focusNode)
			.on("mouseout", blurNode)
//  		.on("dblclick", d3.select(this).classed("fixed", false))
  		.call(drag(simulation));
  
  const nodeLbl = node.append("text")
  			.text(d => d.id)
  			.attr("dy", "0.3em");
  
 	simulation.on("tick", () => {
    link
 			.attr("x1", d => d.source.x)
 			.attr("y1", d => d.source.y)
 			.attr("x2", d => d.target.x)
 			.attr("y2", d => d.target.y);
    node
    	.attr("transform", d => "translate(" + d.x + ", " + d.y + ")");
  });
  
	function focusNode() {
	  d3.select(this).classed("nodeHover", true);
	}

	function blurNode() {
		d3.select(this).classed("nodeHover", false);
	}

});
